<?php
// $Id: sunmailer.mail.inc,v 1.1.2.27 2010/01/13 21:16:51 mwacker Exp $

/**
 * @file
 * Edit, preview and send newsletter, send custom email and statistics.
 */

/**
 * Generates a form for newsletter settings.
 */
function sunmailer_mail_settings() {
  $cutoff = variable_get('sunmailer_cutoff', 0);
  $form = array();
  $form['sunmailer_cutoff'] = array(
    '#title' => t('Cutoff date'),
    '#description' => t('The newsletter will exclude content published before this date'),
    '#required' => TRUE,
    '#default_value' => format_date($cutoff, 'custom', 'Y-m-d H:i'),
  );
  if (module_exists('date_api')) {
    $form['sunmailer_cutoff']['#type'] = module_exists('date_popup') ? 'date_popup' : 'date_select';
    $form['sunmailer_cutoff']['#format'] = 'Y-m-d H:i';
  }
  else {
    $form['sunmailer_cutoff']['#type'] = 'textfield';
    $form['sunmailer_cutoff']['#element_validate'] = array('_sunmailer_date_validate');
  }
  $form['sunmailer_message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#description' => t('This message will appear in the header, with HTML tags stripped for text newsletters'),
    '#default_value' => variable_get('sunmailer_message', ''),
  );
  $form['sunmailer_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Validation callback for default SunMailer date field.
 */
function _sunmailer_date_validate($element, &$form_state) {
  if (!strtotime($element['#value'])) {
    form_error($element, t('Invalid date format.'));
  }
}

/**
 * Submission callback for newsletter settings form.
 */
function sunmailer_mail_settings_submit($form, &$form_state) {
  $values = $form_state['values'];
  variable_set('sunmailer_message', filter_xss(trim($values['sunmailer_message'])));
  variable_set('sunmailer_cutoff', strtotime($values['sunmailer_cutoff']));
}

/**
 * Checks the newsletters for errors.
 */
function sunmailer_check_newsletter() {
  module_load_include('inc', 'sunmailer', 'sunmailer.xml');
  $errors = array();
  // Check the views.
  $sections = sunmailer_get_sections();
  foreach ($sections as $section) {
    $view_name = $section['view'];
    $view = views_get_view($view_name);
    if (!$view || $view->disabled || ($view->base_table != 'node')) {
      $errors[] = t('View !name does not exist, is not a node view, or is disabled.', array('!name' => $view_name));
    }
  }
  if (count($errors)) {
    return $errors;
  }
  // Check the XSL files.
  $xml = sunmailer_xml_newsletter(array_keys(sunmailer_get_sections()));
  if (!$xml) {
    return array();
  }
  $formats = sunmailer_get_formats();
  foreach ($formats as $format) {
    $row = db_fetch_object(db_query("SELECT filename, filepath FROM {files} WHERE fid = %d", $format['fid']));
    $xsl_doc = new DOMDocument();
    if (!$xsl_doc->load($row->filepath)) {
      $errors[] = t('!name is not a valid XSL file.', array('!name' => $row->filename));
      continue;
    }
    $xslt = new XSLTProcessor();
    $xslt->importStyleSheet($xsl_doc);
    if (!$xslt->transformToXml($xml)) {
      $errors[] = t('An XSLT error occurred when transforming the XML with !name.', array('!name' => $row->filename));
    }
  }
  return $errors;
}

/**
 * Displays a preview of the newsletter.
 */
function sunmailer_newsletter_preview($xid) {
  module_load_include('inc', 'sunmailer', 'sunmailer.xml');
  $xml = sunmailer_xml_newsletter(array_keys(sunmailer_get_sections()));
  if (!$xml) {
    print '<p>The email newsletter currently has no content</p>';
    exit;
  }
  $formats = sunmailer_get_formats();
  $format = $formats[$xid];
  $preview = sunmailer_transform_xml($xml, $format);
  if (!$format['html']) {
    $preview = nl2br(htmlentities($preview));
  }
  print $preview;
  exit;
}

/**
 * Checks the last time SunMailer was sent.
 */
function sunmailer_check_last_sent($notify) {
  $last_sent = variable_get('sunmailer_last_sent', 0);
  if (!$last_sent) {
    return TRUE;
  }
  $last_delta = time() - $last_sent;
  $delay = 60 * variable_get('sunmailer_delay', 0);
  if ($last_delta > $delay) {
    if ($notify) {
      drupal_set_message(t('An email was last sent @date.', array('@date' => date('l, F j, Y g:i:s A', $last_sent))));
    }
    return TRUE;
  }
  if ($notify) {
    $message = t('An email was sent @delta seconds ago.  Please wait @delay seconds before sending another.', array('@delta' => $last_delta, '@delay' => $delay));
    drupal_set_message($message, 'warning');
  }
  return FALSE;
}

/**
 * Generates a form to send the newsletter.
 */
function sunmailer_mail_newsletter() {
  module_load_include('inc', 'sunmailer', 'sunmailer.xml');
  // Check for any newsletter errors.
  $errors = sunmailer_check_newsletter();
  if (count($errors)) {
    drupal_set_title('Error');
    $html = '';
    foreach ($errors as $error) {
      $html .= "<p>$error</p>";
    }
    return array('error' => array('#value' => $html));
  }
  // Present a preview of the newsletter in every format.
  $xml = sunmailer_xml_newsletter(array_keys(sunmailer_get_sections()));
  if (!$xml) {
    drupal_set_message(t('The email newsletter currently has no content.'));
    return;
  }
  $formats = sunmailer_get_formats();
  $format = $formats[SUNMAILER_TEXT_FORMAT];
  $preview .= "<p><strong>{$format['name']} preview</strong></p>";
  $preview .= nl2br(htmlentities(sunmailer_transform_xml($xml, $format)));
  $preview .= '<hr /><ul>';
  foreach ($formats as $xid => $format) {
    if ($xid != SUNMAILER_TEXT_FORMAT) {
      $preview .= '<li>' . l("{$format['name']} preview", "sunmailer/preview/$xid") . '</li>';
    }
  }
  $preview .= '</ul><hr />';
  // Check when an email was last sent.
  $can_send = sunmailer_check_last_sent(TRUE);
  // Build the form.
  $form = array();
  $form['text'] = array(
    '#prefix' => '<div>',
    '#value' => $preview,
    '#suffix' => '</div>',
  );
  $message = variable_get('sunmailer_message', '');
  if ($message) {
    $form['delete_message'] = array(
      '#type' => 'checkbox',
      '#title' => t('Delete the message in the header for future newsletters'),
    );
  }
  if (!$can_send) {
    $form['override'] = array(
      '#type' => 'checkbox',
      '#title' => t('Override the time limit to send another email'),
    );
  }
  $form['confirm'] = array(
    '#type' => 'checkbox',
    '#title' => t('Confirm that you want to send the newsletter'),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Send'),
  );
  return $form;
}

/**
 * Validation callback for form to send the newsletter.
 */
function sunmailer_mail_newsletter_validate($form, &$form_state) {
  $values = $form_state['values'];
  if (!$values['confirm']) {
    form_set_error('confirm', t('Cannot send without confirmation.'));
  }
  if (!sunmailer_check_last_sent(FALSE) && !$values['override']) {
    form_set_error('override', t('An email has been sent recently.'));
  }
}

/**
 * Submission callback for form to send the newsletter.
 */
function sunmailer_mail_newsletter_submit($form, &$form_state) {
  module_load_include('inc', 'sunmailer', 'sunmailer.xml');
  $values = $form_state['values'];
  // Perform time-based logic.
  variable_set('sunmailer_last_sent', time());
  variable_set('sunmailer_cutoff', time());
  if ($values['override']) {
    watchdog('mail', 'Overriding SunMailer\'s time limit for sending another email');
  }
  // Set up the mailer.
  watchdog('mail', 'Sending the email newsletter');
  $mailer = sunmailer_get_mailer();
  $subject = variable_get('sunmailer_subject', variable_get('site_name', t('SunMailer')));
  // Build the newsletters for each combination of sections.
  $formats = sunmailer_get_formats();
  $alt_format = $formats[variable_get('sunmailer_text_default', SUNMAILER_TEXT_FORMAT)];
  $subscriptions = sunmailer_get_subscriptions();
  foreach ($subscriptions as $sid_list => $groups) {
    $sids = explode(',', $sid_list);
    $xml = sunmailer_xml_newsletter($sids);
    if (!$xml) {
      continue;
    }
    // Construct the newsletter in each format and send it.
    foreach ($groups as $xid => $addresses) {
      $format = $formats[$xid];
      $body = sunmailer_transform_xml($xml, $format);
      $is_html = $format['html'];
      if ($is_html) {
        $alt_notice = 'Note: You are viewing this as text because your email client does not support HTML.';
        $alt_body = "$alt_notice\n\n" . sunmailer_transform_xml($xml, $alt_format);
      }
      sunmailer_send_mail($mailer, $addresses, $subject, $body, $is_html, $alt_body);
    }
  }
  watchdog('mail', 'Sent the email newsletter');
  // Optionally delete message and redirect.
  if ($form_state['values']['delete_message']) {
    variable_del('sunmailer_message');
  }
  $form_state['redirect'] = user_access('access site reports') ? 'admin/reports/dblog' : 'admin/settings/sunmailer';
}

/**
 * Creates a form to send a custom text email message.
 */
function sunmailer_mail_custom(&$form_state = NULL) {
  // Check when an email was last sent.
  $can_send = sunmailer_check_last_sent(TRUE);
  // Build the form.
  $form = array();
  if (isset($form_state['storage']['preview'])) {
    $form['#prefix'] = $form_state['storage']['preview'];
  }
  $form['message'] = array(
    '#type' => 'fieldset',
    '#title' => t('Custom message'),
    '#description' => t('Message to email to all subscribers'),
  );
  $form['message']['subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => isset($form_state) ? $form_state['values']['subject'] : '',
  );
  $form['message']['body'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#default_value' => isset($form_state) ? $form_state['values']['body'] : '',
  );
  if (!$can_send) {
    $form['override'] = array(
      '#type' => 'checkbox',
      '#title' => t('Override the time limit to send another email'),
    );
  }
  $form['confirm'] = array(
    '#type' => 'checkbox',
    '#title' => t('Confirm that you want to send this email'),
    '#default_value' => FALSE,
  );
  $form['preview'] = array(
    '#type' => 'submit',
    '#value' => t('Preview'),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Send'),
  );
  return $form;
}

/**
 * Validation callback for the form to send a custom email.
 */
function sunmailer_mail_custom_validate($form, &$form_state) {
  $values = $form_state['values'];
  if (!$values['subject']) {
    form_set_error('subject', t('Subject is blank.'));
  }
  if (!$values['body']) {
    form_set_error('body', t('Body is blank.'));
  }
  if ($form_state['clicked_button']['#value'] != t('Preview')) {
    if (!$values['confirm']) {
      form_set_error('confirm', t('Cannot send without confirmation.'));
    }
    if (!sunmailer_check_last_sent(FALSE) && !$values['override']) {
      form_set_error('override', t('An email has been sent recently.'));
    }
  }
}

/**
 * Submission callback for the form to send a custom email.
 */
function sunmailer_mail_custom_submit($form, &$form_state) {
  $values = $form_state['values'];
  // Send the email in text.
  if ($form_state['clicked_button']['#value'] == t('Send')) {
    // Perform time-based logic.
    variable_set('sunmailer_last_sent', time());
    if ($values['override']) {
      watchdog('mail', 'Overriding SunMailer\'s time limit for sending another email');
    }
    // Send the email.
    watchdog('mail', 'Sending a custom email message');
    $mailer = sunmailer_get_mailer();
    $addresses = sunmailer_get_all_subscribers();
    sunmailer_send_mail($mailer, $addresses, check_plain($values['subject']), check_plain($values['body']));
    watchdog('mail', 'Sent a custom email message');
    // Redirect.
    unset($form_state['storage']);
    $form_state['redirect'] = user_access('access site reports') ? 'admin/reports/dblog' : 'admin/settings/sunmailer';
  }
  // Store a preview of the email newsletter.
  else {
    $preview = "<h3>" . check_plain($form_state['values']['subject']) . "</h3>";
    $preview .= "<p>" . check_plain($form_state['values']['body']) . "</p>";
    $form_state['storage']['preview'] = $preview;
  }
}

/**
 * Calculates subscription statistics.
 */
function get_sunmailer_stats() {
  // Initialize the statistics.
  $sections = sunmailer_get_sections();
  $formats = sunmailer_get_formats();
  $stats['total'] = 0;
  $stats['splits']['section'] = array();
  foreach ($sections as $section) {
    $stats['splits']['section'][$section['name']] = 0;
  }
  $stats['splits']['format'] = array();
  foreach ($formats as $format) {
    $stats['splits']['format'][$format['name']] = 0;
  }
  // Calculate the statistics.
  $subscriptions = sunmailer_get_subscriptions();
  foreach ($subscriptions as $sid_list => $sections_group) {
    $sids = explode(',', $sid_list);
    foreach ($sections_group as $xid => $format_group) {
      $count = count($format_group);
      $stats['total'] += $count;
      $stats['splits']['format'][$formats[$xid]['name']] += $count;
      foreach ($sids as $sid) {
        $stats['splits']['section'][$sections[$sid]['name']] += $count;
      }
    }
  }
  return $stats;
}

/**
 * Displays a page of statistics.
 */
function sunmailer_stats_page() {
  return theme('sunmailer_stats', get_sunmailer_stats());
}

/**
 * Default theme for statistics.
 */
function theme_sunmailer_stats($stats) {
  $html = '<h3>' . t('Total subscribers') . '</h3>';
  $html .= '<ul><li>' . check_plain($stats['total']) . '</li></ul>';
  foreach ($stats['splits'] as $name => $split) {
    $html .= '<h4>' . t('By @name', array('@name' => $name)) . '</h4>';
    $html .= '<ul>';
    foreach ($split as $name => $count) {
      $html .= '<li>' . t('@name: @count', array('@name' => $name, '@count' => $count)) . '</li>';
    }
    $html .= '</ul>';
  }
  return $html;
}
